@@ -11,6 +11,9 @@ public:
 	COpData(Command op_Id);
 	virtual ~COpData();
 
+	COpData(COpData const&) = delete;
+	COpData& operator=(COpData const&) = delete;
+
 	int opState;
 	Command const opId;
 
@@ -24,8 +27,8 @@ class CConnectOpData : public COpData
 {
 public:
 	CConnectOpData()
-		: COpData(Command::connect),
-		port(0)
+		: COpData(Command::connect)
+		, port(0)
 	{
 	}
 
@@ -44,20 +47,20 @@ public:
 	const bool download;
 
 	CDateTime fileTime;
-	wxFileOffset localFileSize;
-	wxFileOffset remoteFileSize;
+	wxFileOffset localFileSize{-1};
+	wxFileOffset remoteFileSize{-1};
 
-	bool tryAbsolutePath;
-	bool resume;
+	bool tryAbsolutePath{};
+	bool resume{};
 
 	CFileTransferCommand::t_transferSettings transferSettings;
 
 	// Set to true when sending the command which
 	// starts the actual transfer
-	bool transferInitiated;
+	bool transferInitiated{};
 };
 
-class CMkdirOpData : public COpData
+class CMkdirOpData final : public COpData
 {
 public:
 	CMkdirOpData()
@@ -65,14 +68,10 @@ public:
 	{
 	}
 
-	virtual ~CMkdirOpData()
-	{
-	}
-
 	CServerPath path;
 	CServerPath currentPath;
 	CServerPath commonParent;
-	std::list<wxString> segments;
+	std::vector<wxString> segments;
 };
 
 class CChangeDirOpData : public COpData
@@ -112,12 +111,15 @@ enum class TransferEndReason
 };
 
 class CTransferStatus;
-class CControlSocket: public wxEvtHandler, public CLogging
+class CControlSocket: public CLogging, public CEventHandler
 {
 public:
-	CControlSocket(CFileZillaEnginePrivate *pEngine);
+	CControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CControlSocket();
 
+	CControlSocket(CControlSocket const&) = delete;
+	CControlSocket& operator=(CControlSocket const&) = delete;
+
 	virtual int Connect(const CServer &server) = 0;
 	virtual int Disconnect();
 	virtual void Cancel();
@@ -126,7 +128,7 @@ public:
 							 const wxString &remoteFile, bool download,
 							 const CFileTransferCommand::t_transferSettings& transferSettings);
 	virtual int RawCommand(const wxString& command = wxString());
-	virtual int Delete(const CServerPath& path, const std::list<wxString>& files);
+	virtual int Delete(const CServerPath& path, std::deque<wxString>&& files);
 	virtual int RemoveDir(const CServerPath& path = CServerPath(), const wxString& subDir = wxString());
 	virtual int Mkdir(const CServerPath& path);
 	virtual int Rename(const CRenameCommand& command);
@@ -143,21 +145,16 @@ public:
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification) = 0;
 	bool SetFileExistsAction(CFileExistsNotification *pFileExistsNotification);
 
-	void InitTransferStatus(wxFileOffset totalSize, wxFileOffset startOffset, bool list);
-	void SetTransferStatusStartTime();
-	void UpdateTransferStatus(wxFileOffset transferredBytes);
-	void ResetTransferStatus();
-	bool GetTransferStatus(CTransferStatus &status, bool &changed);
-	void SetTransferStatusMadeProgress();
-
 	const CServer* GetCurrentServer() const;
 
 	// Conversion function which convert between local and server charset.
-	wxString ConvToLocal(const char* buffer);
-	wxChar* ConvToLocalBuffer(const char* buffer);
-	wxChar* ConvToLocalBuffer(const char* buffer, wxMBConv& conv);
+	wxString ConvToLocal(const char* buffer, size_t len);
+	wxChar* ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen);
+	wxChar* ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen);
 	wxCharBuffer ConvToServer(const wxString& str, bool force_utf8 = false);
 
+	void SetActive(CFileZillaEngine::_direction direction);
+
 	// ---
 	// The following two functions control the timeout behaviour:
 	// ---
@@ -168,13 +165,13 @@ public:
 	// Set to true if waiting for data
 	void SetWait(bool waiting);
 
-	CFileZillaEnginePrivate* GetEngine() { return m_pEngine; }
+	CFileZillaEnginePrivate& GetEngine() { return engine_; }
 
 	// Only called from the engine, see there for description
 	void InvalidateCurrentWorkingDir(const CServerPath& path);
 
 protected:
-	wxTimeSpan GetTimezoneOffset();
+	duration GetTimezoneOffset() const;
 
 	virtual int DoClose(int nErrorCode = FZ_REPLY_DISCONNECTED);
 	bool m_closed;
@@ -198,20 +195,17 @@ protected:
 
 	COpData *m_pCurOpData;
 	int m_nOpState;
-	CFileZillaEnginePrivate *m_pEngine;
+	CFileZillaEnginePrivate & engine_;
 	CServer *m_pCurrentServer;
 
 	CServerPath m_CurrentPath;
 
-	CTransferStatus *m_pTransferStatus;
-	int m_transferStatusSendState;
-
 	wxCSConv *m_pCSConv;
 	bool m_useUTF8;
 
 	// Timeout data
-	wxTimer m_timer;
-	wxStopWatch m_stopWatch;
+	timer_id m_timer{};
+	CMonotonicClock m_lastActivity;
 
 	// -------------------------
 	// Begin cache locking stuff
@@ -244,10 +238,6 @@ protected:
 
 	bool IsWaitingForLock();
 
-#ifdef __VISUALC__
-	// Retarded compiler does not like my code
-	public:
-#endif
 	struct t_lockInfo
 	{
 		CControlSocket* pControlSocket;
@@ -257,9 +247,6 @@ protected:
 		int lockcount;
 	};
 	static std::list<t_lockInfo> m_lockInfoList;
-#ifdef __VISUALC__
-	protected:
-#endif
 
 	const std::list<t_lockInfo>::iterator GetLockStatus();
 
@@ -269,16 +256,17 @@ protected:
 
 	bool m_invalidateCurrentPath;
 
-	DECLARE_EVENT_TABLE()
-	void OnTimer(wxTimerEvent& event);
-	void OnObtainLock(wxCommandEvent& event);
+	virtual void operator()(CEventBase const& ev);
+
+	void OnTimer(timer_id id);
+	void OnObtainLock();
 };
 
 class CProxySocket;
-class CRealControlSocket : public CControlSocket, public CSocketEventHandler
+class CRealControlSocket : public CControlSocket
 {
 public:
-	CRealControlSocket(CFileZillaEnginePrivate *pEngine);
+	CRealControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CRealControlSocket();
 
 	virtual int Connect(const CServer &server);
@@ -290,10 +278,13 @@ protected:
 	virtual int DoClose(int nErrorCode = FZ_REPLY_DISCONNECTED);
 	void ResetSocket();
 
-	virtual void OnSocketEvent(CSocketEvent &event);
+	virtual void operator()(CEventBase const& ev);
+	void OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error);
+	void OnHostAddress(CSocketEventSource* source, wxString const& address);
+
 	virtual void OnConnect();
 	virtual void OnReceive();
-	virtual void OnSend();
+	void OnSend();
 	virtual void OnClose(int error);
 
 	bool Send(const char *buffer, int len);
