@@ -14,9 +14,11 @@ but for some operations the engine/interface prefers to retrieve a clean
 version.
 */
 
-const int CACHE_TIMEOUT = 1800; // In seconds
+#include <mutex.h>
 
-class CDirectoryCache
+const int CACHE_TIMEOUT = 900; // In seconds
+
+class CDirectoryCache final
 {
 public:
 	enum Filetype
@@ -29,8 +31,11 @@ public:
 	CDirectoryCache();
 	~CDirectoryCache();
 
+	CDirectoryCache(CDirectoryCache const&) = delete;
+	CDirectoryCache& operator=(CDirectoryCache const&) = delete;
+
 	void Store(const CDirectoryListing &listing, const CServer &server);
-	bool GetChangeTime(CMonotonicTime& time, const CServer &server, const CServerPath &path);
+	bool GetChangeTime(CMonotonicClock& time, const CServer &server, const CServerPath &path);
 	bool Lookup(CDirectoryListing &listing, const CServer &server, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated);
 	bool DoesExist(const CServer &server, const CServerPath &path, int &hasUnsureEntries, bool &is_outdated);
 	bool LookupFile(CDirentry &entry, const CServer &server, const CServerPath &path, const wxString& file, bool &dirDidExist, bool &matchedCase);
@@ -43,27 +48,26 @@ public:
 
 protected:
 
-	class CCacheEntry
+	class CCacheEntry final
 	{
 	public:
 		CCacheEntry() : lruIt() { };
 		CCacheEntry(const CCacheEntry &entry);
 		explicit CCacheEntry(CDirectoryListing const& l)
 			: listing(l)
-			, modificationTime(CMonotonicTime::Now())
-			, lruIt()
+			, modificationTime(CMonotonicClock::now())
 		{}
 		~CCacheEntry() { };
 
 		CDirectoryListing listing;
-		CMonotonicTime modificationTime;
+		CMonotonicClock modificationTime;
 
 		CCacheEntry& operator=(const CCacheEntry &a);
 
-		void* lruIt; // void* to break cyclic declaration dependency
+		void* lruIt{}; // void* to break cyclic declaration dependency
 	};
 
-	class CServerEntry
+	class CServerEntry final
 	{
 	public:
 		CServerEntry() {}
@@ -85,9 +89,9 @@ protected:
 
 	bool Lookup(tCacheIter &cacheIter, tServerIter &sit, const CServerPath &path, bool allowUnsureEntries, bool& is_outdated);
 
-	static std::list<CServerEntry> m_serverList;
+	mutex mutex_;
 
-	static int m_nRefCount;
+	std::list<CServerEntry> m_serverList;
 
 	void UpdateLru(tServerIter const& sit, tCacheIter const& cit);
 
@@ -95,9 +99,9 @@ protected:
 
 	typedef std::pair<tServerIter, tCacheIter> tFullEntryPosition;
 	typedef std::list<tFullEntryPosition> tLruList;
-	static tLruList m_leastRecentlyUsedList;
+	tLruList m_leastRecentlyUsedList;
 
-	static wxLongLongNative m_totalFileCount;
+	int64_t m_totalFileCount{};
 };
 
 #endif
