@@ -1,14 +1,15 @@
 #include <filezilla.h>
 
 CDirectoryListing::CDirectoryListing()
+	: m_flags()
+	, m_entryCount()
 {
-	m_entryCount = 0;
-	m_flags = 0;
 }
 
 CDirectoryListing::CDirectoryListing(const CDirectoryListing& listing)
 	: path(listing.path)
-	, m_flags(listing.m_flags), m_firstListTime(listing.m_firstListTime)
+	, m_firstListTime(listing.m_firstListTime)
+	, m_flags(listing.m_flags)
 	, m_entries(listing.m_entries), m_searchmap_case(listing.m_searchmap_case), m_searchmap_nocase(listing.m_searchmap_nocase)
 	, m_entryCount(listing.m_entryCount)
 {
@@ -42,10 +43,10 @@ wxString CDirentry::dump() const
 				target ? *target : wxString(), flags & flag_unsure);
 
 	if( has_date() ) {
-		str += _T("date=") + time.Degenerate().FormatISODate() + _T("\n");
+		str += _T("date=") + time.Format(_T("%Y-%m-%d"), CDateTime::local) + _T("\n");
 	}
 	if( has_time() ) {
-		str += _T("time=") + time.Degenerate().FormatISOTime() + _T("\n");
+		str += _T("time=") + time.Format(_T("%H-%M-%S"), CDateTime::local) + _T("\n");
 	}
 	return str;
 }
@@ -113,7 +114,7 @@ CDirentry& CDirectoryListing::operator[](unsigned int index)
 	return m_entries.Get()[index].Get();
 }
 
-void CDirectoryListing::Assign(const std::list<CDirentry> &entries)
+void CDirectoryListing::Assign(std::deque<CRefcountObject<CDirentry>> &entries)
 {
 	m_entryCount = entries.size();
 
@@ -123,15 +124,14 @@ void CDirectoryListing::Assign(const std::list<CDirentry> &entries)
 
 	m_flags &= ~(listing_has_dirs | listing_has_perms | listing_has_usergroup);
 
-	for (std::list<CDirentry>::const_iterator iter = entries.begin(); iter != entries.end(); ++iter)
-	{
-		if (iter->is_dir())
+	for (auto & entry : entries) {
+		if (entry->is_dir())
 			m_flags |= listing_has_dirs;
-		if (!iter->permissions->empty())
+		if (!entry->permissions->empty())
 			m_flags |= listing_has_perms;
-		if (!iter->ownerGroup->empty())
+		if (!entry->ownerGroup->empty())
 			m_flags |= listing_has_usergroup;
-		own_entries.push_back(CRefcountObject<CDirentry>(*iter));
+		own_entries.emplace_back(std::move(entry));
 	}
 
 	m_searchmap_case.clear();
