@@ -1,70 +1,34 @@
 #include <filezilla.h>
-#include "transfersocket.h"
-#include "ftpcontrolsocket.h"
 #include "directorylistingparser.h"
-#include "optionsbase.h"
+#include "engineprivate.h"
+#include "ftpcontrolsocket.h"
 #include "iothread.h"
+#include "optionsbase.h"
 #include "tlssocket.h"
-#include <errno.h>
+#include "transfersocket.h"
 #include "proxy.h"
 #include "servercapabilities.h"
 
-BEGIN_EVENT_TABLE(CTransferSocket, wxEvtHandler)
-	EVT_IOTHREAD(wxID_ANY, CTransferSocket::OnIOThreadEvent)
-END_EVENT_TABLE()
-
-CTransferSocket::CTransferSocket(CFileZillaEnginePrivate *pEngine, CFtpControlSocket *pControlSocket, TransferMode transferMode)
+CTransferSocket::CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSocket & controlSocket, TransferMode transferMode)
+: CEventHandler(controlSocket.event_loop_)
+, engine_(engine)
+, controlSocket_(controlSocket)
+, m_transferMode(transferMode)
 {
-	m_pEngine = pEngine;
-	m_pControlSocket = pControlSocket;
-
-	m_pSocketServer = 0;
-	m_pSocket = 0;
-	m_pBackend = 0;
-	m_pProxyBackend = 0;
-	m_pTlsSocket = 0;
-
-	m_pDirectoryListingParser = 0;
-
-	m_bActive = false;
-
-	SetEvtHandlerEnabled(true);
-
-	m_transferMode = transferMode;
-
-	m_pTransferBuffer = 0;
-	m_transferBufferLen = 0;
-
-	m_transferEndReason = TransferEndReason::none;
-	m_binaryMode = true;
-
-	m_onCloseCalled = false;
-
-	m_postponedReceive = false;
-	m_postponedSend = false;
-
-	m_shutdown = false;
-
-	m_madeProgress = 0;
 }
 
 CTransferSocket::~CTransferSocket()
 {
+	RemoveHandler();
 	if (m_transferEndReason == TransferEndReason::none)
 		m_transferEndReason = TransferEndReason::successful;
 	ResetSocket();
 
-	if (m_pControlSocket)
-	{
-		if (m_transferMode == TransferMode::upload || m_transferMode == TransferMode::download)
-		{
-			CFtpFileTransferOpData *pData = static_cast<CFtpFileTransferOpData *>(static_cast<CRawTransferOpData *>(m_pControlSocket->m_pCurOpData)->pOldData);
-			if (pData && pData->pIOThread)
-			{
-				if (m_transferMode == TransferMode::download)
-					FinalizeWrite();
-				pData->pIOThread->SetEventHandler(0);
-			}
+	if (m_transferMode == TransferMode::upload || m_transferMode == TransferMode::download) {
+		if (ioThread_) {
+			if (m_transferMode == TransferMode::download)
+				FinalizeWrite();
+			ioThread_->SetEventHandler(0);
 		}
 	}
 }
@@ -93,7 +57,7 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 	m_pSocketServer = CreateSocketServer();
 
 	if (!m_pSocketServer) {
-		m_pControlSocket->LogMessage(MessageType::Debug_Warning, _T("CreateSocketServer failed"));
+		controlSocket_.LogMessage(MessageType::Debug_Warning, _T("CreateSocketServer failed"));
 		return wxString();
 	}
 
@@ -102,10 +66,18 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 	if (port == -1)	{
 		ResetSocket();
 
-		m_pControlSocket->LogMessage(MessageType::Debug_Warning, _T("GetLocalPort failed: %s"), CSocket::GetErrorDescription(error));
+		controlSocket_.LogMessage(MessageType::Debug_Warning, _T("GetLocalPort failed: %s"), CSocket::GetErrorDescription(error));
 		return wxString();
 	}
 
+	if (engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS)) {
+		port += static_cast<int>(engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS_OFFSET));
+		if (port <= 0 || port >= 65536) {
+			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Port outside valid range"));
+			return wxString();
+		}
+	}
+
 	wxString portArguments;
 	if (m_pSocketServer->GetAddressFamily() == CSocket::ipv6) {
 		portArguments = wxString::Format(_T("|2|%s|%d|"), ip, port);
@@ -119,17 +91,16 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 	return portArguments;
 }
 
-void CTransferSocket::OnSocketEvent(CSocketEvent &event)
+void CTransferSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
 	if (m_pProxyBackend)
 	{
-		switch (event.GetType())
+		switch (t)
 		{
-		case CSocketEvent::connection:
+		case SocketEventType::connection:
 			{
-				const int error = event.GetError();
 				if (error) {
-					m_pControlSocket->LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error));
+					controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error));
 					TransferEnd(TransferEndReason::failure);
 				}
 				else {
@@ -139,66 +110,62 @@ void CTransferSocket::OnSocketEvent(CSocketEvent &event)
 				}
 			}
 			return;
-		case CSocketEvent::close:
+		case SocketEventType::close:
 			{
-				const int error = event.GetError();
-				m_pControlSocket->LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error));
+				controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error));
 				TransferEnd(TransferEndReason::failure);
 			}
 			return;
 		default:
 			// Uninteresting
-			return;
+			break;
 		}
 		return;
 	}
 
 	if (m_pSocketServer) {
-		if (event.GetType() == CSocketEvent::connection)
-			OnAccept(event.GetError());
+		if (t == SocketEventType::connection)
+			OnAccept(error);
 		else
-			m_pControlSocket->LogMessage(MessageType::Debug_Info, _T("Unhandled socket event %d from listening socket"), event.GetType());
+			controlSocket_.LogMessage(MessageType::Debug_Info, _T("Unhandled socket event %d from listening socket"), t);
 		return;
 	}
 
-	switch (event.GetType())
+	switch (t)
 	{
-	case CSocketEvent::connection:
-		if (event.GetError()) {
+	case SocketEventType::connection:
+		if (error) {
 			if (m_transferEndReason == TransferEndReason::none) {
-				m_pControlSocket->LogMessage(MessageType::Error, _("The data connection could not be established: %s"), CSocket::GetErrorDescription(event.GetError()));
+				controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), CSocket::GetErrorDescription(error));
 				TransferEnd(TransferEndReason::transfer_failure);
 			}
 		}
 		else
 			OnConnect();
 		break;
-	case CSocketEvent::read:
+	case SocketEventType::read:
 		OnReceive();
 		break;
-	case CSocketEvent::write:
+	case SocketEventType::write:
 		OnSend();
 		break;
-	case CSocketEvent::close:
-		OnClose(event.GetError());
-		break;
-	case CSocketEvent::hostaddress:
-		// Booooring. No seriously, we connect by IP already, nothing to resolve
+	case SocketEventType::close:
+		OnClose(error);
 		break;
 	default:
-		m_pControlSocket->LogMessage(MessageType::Debug_Info, _T("Unhandled socket event %d"), event.GetType());
+		// Uninteresting
 		break;
 	}
 }
 
 void CTransferSocket::OnAccept(int error)
 {
-	m_pControlSocket->SetAlive();
-	m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnAccept(%d)"), error);
+	controlSocket_.SetAlive();
+	controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnAccept(%d)"), error);
 
 	if (!m_pSocketServer)
 	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Warning, _T("No socket server in OnAccept"), error);
+		controlSocket_.LogMessage(MessageType::Debug_Warning, _T("No socket server in OnAccept"), error);
 		return;
 	}
 
@@ -206,9 +173,9 @@ void CTransferSocket::OnAccept(int error)
 	if (!m_pSocket)
 	{
 		if (error == EAGAIN)
-			m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("No pending connection"));
+			controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("No pending connection"));
 		else {
-			m_pControlSocket->LogMessage(MessageType::Status, _("Could not accept connection: %s"), CSocket::GetErrorDescription(error));
+			controlSocket_.LogMessage(MessageType::Status, _("Could not accept connection: %s"), CSocket::GetErrorDescription(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 		return;
@@ -221,12 +188,12 @@ void CTransferSocket::OnAccept(int error)
 
 void CTransferSocket::OnConnect()
 {
-	m_pControlSocket->SetAlive();
-	m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnConnect"));
+	controlSocket_.SetAlive();
+	controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnConnect"));
 
 	if (!m_pSocket)
 	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnConnect called without socket"));
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnConnect called without socket"));
 		return;
 	}
 
@@ -241,8 +208,8 @@ void CTransferSocket::OnConnect()
 	else if (m_pTlsSocket) {
 		// Re-enable Nagle algorithm
 		m_pSocket->SetFlags(m_pSocket->GetFlags() & (~CSocket::flag_nodelay));
-		if (CServerCapabilities::GetCapability(*m_pControlSocket->m_pCurrentServer, tls_resume) == unknown)	{
-			CServerCapabilities::SetCapability(*m_pControlSocket->m_pCurrentServer, tls_resume, m_pTlsSocket->ResumedSession() ? yes : no);
+		if (CServerCapabilities::GetCapability(*controlSocket_.m_pCurrentServer, tls_resume) == unknown)	{
+			CServerCapabilities::SetCapability(*controlSocket_.m_pCurrentServer, tls_resume, m_pTlsSocket->ResumedSession() ? yes : no);
 		}
 	}
 
@@ -252,151 +219,135 @@ void CTransferSocket::OnConnect()
 
 void CTransferSocket::OnReceive()
 {
-	m_pControlSocket->LogMessage(MessageType::Debug_Debug, _T("CTransferSocket::OnReceive(), m_transferMode=%d"), m_transferMode);
+	controlSocket_.LogMessage(MessageType::Debug_Debug, _T("CTransferSocket::OnReceive(), m_transferMode=%d"), m_transferMode);
 
-	if (!m_pBackend)
-	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("Postponing receive, m_pBackend was false."));
+	if (!m_pBackend) {
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Postponing receive, m_pBackend was false."));
 		m_postponedReceive = true;
 		return;
 	}
 
-	if (!m_bActive)
-	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("Postponing receive, m_bActive was false."));
+	if (!m_bActive) {
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Postponing receive, m_bActive was false."));
 		m_postponedReceive = true;
 		return;
 	}
 
-	if (m_transferMode == TransferMode::list)
-	{
-		for (;;)
-		{
+	if (m_transferMode == TransferMode::list) {
+		for (;;) {
 			char *pBuffer = new char[4096];
 			int error;
 			int numread = m_pBackend->Read(pBuffer, 4096, error);
-			if (numread < 0)
-			{
+			if (numread < 0) {
 				delete [] pBuffer;
 				if (error != EAGAIN) {
-					m_pControlSocket->LogMessage(MessageType::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error));
+					controlSocket_.LogMessage(MessageType::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
 				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
 					TransferEnd(TransferEndReason::successful);
-
 				return;
 			}
 
-			if (numread > 0)
-			{
+			if (numread > 0) {
 				if (!m_pDirectoryListingParser->AddData(pBuffer, numread))
 				{
 					TransferEnd(TransferEndReason::transfer_failure);
 					return;
 				}
 
-				m_pEngine->SetActive(CFileZillaEngine::recv);
-				if (!m_madeProgress)
-				{
+				controlSocket_.SetActive(CFileZillaEngine::recv);
+				if (!m_madeProgress) {
 					m_madeProgress = 2;
-					m_pControlSocket->SetTransferStatusMadeProgress();
+					engine_.transfer_status_.SetMadeProgress();
 				}
-				m_pControlSocket->UpdateTransferStatus(numread);
+				engine_.transfer_status_.Update(numread);
 			}
-			else
-			{
+			else {
 				delete [] pBuffer;
 				TransferEnd(TransferEndReason::successful);
 				return;
 			}
 		}
 	}
-	else if (m_transferMode == TransferMode::download)
-	{
-		for (;;)
-		{
+	else if (m_transferMode == TransferMode::download) {
+		int error;
+		int numread;
+
+		// Only do a certain number of iterations in one go to keep the event loop going.
+		// Otherwise this behaves like a livelock on very large files written to a very fast
+		// SSD downloaded from a very fast server.
+		for (int i = 0; i < 100; ++i) {
 			if (!CheckGetNextWriteBuffer())
 				return;
 
-			int error;
-			int numread = m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen, error);
-			if (numread < 0)
-			{
-				if (error != EAGAIN) {
-					m_pControlSocket->LogMessage(MessageType::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error));
-					TransferEnd(TransferEndReason::transfer_failure);
-				}
-				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
-					TransferEnd(TransferEndReason::successful);
-				return;
-			}
-
-			if (numread > 0)
-			{
-				m_pEngine->SetActive(CFileZillaEngine::recv);
-				if (!m_madeProgress)
-				{
-					m_madeProgress = 2;
-					m_pControlSocket->SetTransferStatusMadeProgress();
-				}
-				m_pControlSocket->UpdateTransferStatus(numread);
-
-				m_pTransferBuffer += numread;
-				m_transferBufferLen -= numread;
-
-				if (!CheckGetNextWriteBuffer())
-					return;
-			}
-			else //!numread
-			{
-				FinalizeWrite();
+			numread = m_pBackend->Read(m_pTransferBuffer, m_transferBufferLen, error);
+			if (numread <= 0) {
 				break;
 			}
+
+			controlSocket_.SetActive(CFileZillaEngine::recv);
+			if (!m_madeProgress) {
+				m_madeProgress = 2;
+				engine_.transfer_status_.SetMadeProgress();
+			}
+			engine_.transfer_status_.Update(numread);
+
+			m_pTransferBuffer += numread;
+			m_transferBufferLen -= numread;
+		}
+
+		if (numread < 0) {
+			if (error != EAGAIN) {
+				controlSocket_.LogMessage(MessageType::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error));
+				TransferEnd(TransferEndReason::transfer_failure);
+			}
+			else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
+				FinalizeWrite();
+			}
+		}
+		else if (!numread) {
+			FinalizeWrite();
+		}
+		else {
+			SendEvent<CSocketEvent>(m_pBackend, SocketEventType::read, 0);
 		}
 	}
-	else if (m_transferMode == TransferMode::resumetest)
-	{
-		for (;;)
-		{
+	else if (m_transferMode == TransferMode::resumetest) {
+		for (;;) {
 			char buffer[2];
 			int error;
 			int numread = m_pBackend->Read(buffer, 2, error);
-			if (numread < 0)
-			{
+			if (numread < 0) {
 				if (error != EAGAIN) {
-					m_pControlSocket->LogMessage(MessageType::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error));
+					controlSocket_.LogMessage(MessageType::Error, _T("Could not read from transfer socket: %s"), CSocket::GetErrorDescription(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
-				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound))
-				{
+				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
 					if (m_transferBufferLen == 1)
 						TransferEnd(TransferEndReason::successful);
 					else
 					{
-						m_pControlSocket->LogMessage(MessageType::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
+						controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
 						TransferEnd(TransferEndReason::failed_resumetest);
 					}
 				}
 				return;
 			}
 
-			if (!numread)
-			{
+			if (!numread) {
 				if (m_transferBufferLen == 1)
 					TransferEnd(TransferEndReason::successful);
-				else
-				{
-					m_pControlSocket->LogMessage(MessageType::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
+				else {
+					controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
 					TransferEnd(TransferEndReason::failed_resumetest);
 				}
 				return;
 			}
 			m_transferBufferLen += numread;
 
-			if (m_transferBufferLen > 1)
-			{
-				m_pControlSocket->LogMessage(MessageType::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
+			if (m_transferBufferLen > 1) {
+				controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Server incorrectly sent %d bytes"), m_transferBufferLen);
 				TransferEnd(TransferEndReason::failed_resumetest);
 				return;
 			}
@@ -406,15 +357,13 @@ void CTransferSocket::OnReceive()
 
 void CTransferSocket::OnSend()
 {
-	if (!m_pBackend)
-	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("OnSend called without backend. Ignoring event."));
+	if (!m_pBackend) {
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("OnSend called without backend. Ignoring event."));
 		return;
 	}
 
-	if (!m_bActive)
-	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("Postponing send"));
+	if (!m_bActive) {
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Postponing send"));
 		m_postponedSend = true;
 		return;
 	}
@@ -424,8 +373,11 @@ void CTransferSocket::OnSend()
 
 	int error;
 	int written;
-	do
-	{
+
+	// Only do a certain number of iterations in one go to keep the event loop going.
+	// Otherwise this behaves like a livelock on very large files read from a very fast
+	// SSD uploaded to a very fast server.
+	for (int i = 0; i < 100; ++i) {
 		if (!CheckGetNextReadBuffer())
 			return;
 
@@ -433,38 +385,39 @@ void CTransferSocket::OnSend()
 		if (written <= 0)
 			break;
 
-		m_pEngine->SetActive(CFileZillaEngine::send);
-		if (m_madeProgress == 1)
-		{
-			m_pControlSocket->LogMessage(MessageType::Debug_Debug, _T("Made progress in CTransferSocket::OnSend()"));
+		controlSocket_.SetActive(CFileZillaEngine::send);
+		if (m_madeProgress == 1) {
+			controlSocket_.LogMessage(MessageType::Debug_Debug, _T("Made progress in CTransferSocket::OnSend()"));
 			m_madeProgress = 2;
-			m_pControlSocket->SetTransferStatusMadeProgress();
+			engine_.transfer_status_.SetMadeProgress();
 		}
-		m_pControlSocket->UpdateTransferStatus(written);
+		engine_.transfer_status_.Update(written);
 
 		m_pTransferBuffer += written;
 		m_transferBufferLen -= written;
 	}
-	while (true);
 
 	if (written < 0) {
 		if (error == EAGAIN) {
 			if (!m_madeProgress) {
-				m_pControlSocket->LogMessage(MessageType::Debug_Debug, _T("First EAGAIN in CTransferSocket::OnSend()"));
+				controlSocket_.LogMessage(MessageType::Debug_Debug, _T("First EAGAIN in CTransferSocket::OnSend()"));
 				m_madeProgress = 1;
-				m_pControlSocket->SetTransferStatusMadeProgress();
+				engine_.transfer_status_.SetMadeProgress();
 			}
 		}
 		else {
-			m_pControlSocket->LogMessage(MessageType::Error, _T("Could not write to transfer socket: %s"), CSocket::GetErrorDescription(error));
+			controlSocket_.LogMessage(MessageType::Error, _T("Could not write to transfer socket: %s"), CSocket::GetErrorDescription(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 	}
+	else if (written > 0) {
+		SendEvent<CSocketEvent>(m_pBackend, SocketEventType::write, 0);
+	}
 }
 
 void CTransferSocket::OnClose(int error)
 {
-	m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnClose(%d)"), error);
+	controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::OnClose(%d)"), error);
 	m_onCloseCalled = true;
 
 	if (m_transferEndReason != TransferEndReason::none)
@@ -477,10 +430,8 @@ void CTransferSocket::OnClose(int error)
 		}
 	}
 
-	if (m_transferMode == TransferMode::upload)
-	{
-		if (m_shutdown && m_pTlsSocket)
-		{
+	if (m_transferMode == TransferMode::upload) {
+		if (m_shutdown && m_pTlsSocket) {
 			if (m_pTlsSocket->Shutdown() != 0)
 				TransferEnd(TransferEndReason::transfer_failure);
 			else
@@ -492,7 +443,7 @@ void CTransferSocket::OnClose(int error)
 	}
 
 	if (error) {
-		m_pControlSocket->LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error));
+		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error));
 		TransferEnd(TransferEndReason::transfer_failure);
 		return;
 	}
@@ -514,7 +465,7 @@ void CTransferSocket::OnClose(int error)
 		return;
 	}
 	else if (numread < 0 && error != EAGAIN) {
-		m_pControlSocket->LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error));
+		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error));
 		TransferEnd(TransferEndReason::transfer_failure);
 		return;
 	}
@@ -525,7 +476,12 @@ void CTransferSocket::OnClose(int error)
 			return;
 		}
 	}
-	TransferEnd(TransferEndReason::successful);
+	if (m_transferMode == TransferMode::download) {
+		FinalizeWrite();
+	}
+	else {
+		TransferEnd(TransferEndReason::successful);
+	}
 }
 
 bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
@@ -534,24 +490,22 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 
 	m_pSocket = new CSocket(this);
 
-	if (m_pControlSocket->m_pProxyBackend)
-	{
-		m_pProxyBackend = new CProxySocket(this, m_pSocket, m_pControlSocket);
+	if (controlSocket_.m_pProxyBackend) {
+		m_pProxyBackend = new CProxySocket(this, m_pSocket, &controlSocket_);
 
-		int res = m_pProxyBackend->Handshake(m_pControlSocket->m_pProxyBackend->GetProxyType(),
+		int res = m_pProxyBackend->Handshake(controlSocket_.m_pProxyBackend->GetProxyType(),
 											 host, port,
-											 m_pControlSocket->m_pProxyBackend->GetUser(), m_pControlSocket->m_pProxyBackend->GetPass());
+											 controlSocket_.m_pProxyBackend->GetUser(), controlSocket_.m_pProxyBackend->GetPass());
 
-		if (res != EINPROGRESS)
-		{
+		if (res != EINPROGRESS) {
 			ResetSocket();
 			return false;
 		}
 		int error;
-		host = m_pControlSocket->m_pSocket->GetPeerIP();
-		port = m_pControlSocket->m_pSocket->GetRemotePort(error);
-		if( host.empty() || port < 1 ) {
-			m_pControlSocket->LogMessage(MessageType::Debug_Warning, _T("Could not get peer address of control connection."));
+		host = controlSocket_.m_pSocket->GetPeerIP();
+		port = controlSocket_.m_pSocket->GetRemotePort(error);
+		if (host.empty() || port < 1) {
+			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Could not get peer address of control connection."));
 			ResetSocket();
 			return false;
 		}
@@ -559,9 +513,29 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 
 	SetSocketBufferSizes(m_pSocket);
 
-	int res = m_pSocket->Connect(host, port);
-	if (res && res != EINPROGRESS)
-	{
+	// Try to bind the source IP of the data connection to the same IP as the control connection.
+	// We can do so either if 
+	// 1) the destination IP of the data connection matches peer IP of the control connection or
+	// 2) we are using a proxy.
+	//
+	// In case destination IPs of control and data connection are different, do not bind to the
+	// same source.
+
+	wxString bindAddress;
+	if (m_pProxyBackend) {
+		bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+	}
+	else {
+		if (controlSocket_.m_pSocket->GetPeerIP(true) == host || controlSocket_.m_pSocket->GetPeerIP(false) == host) {
+			bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+		}
+		else {
+			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Destination IP of data connection does not match peer IP of control connection. Not binding source address of data connection."));
+		}
+	}
+
+	int res = m_pSocket->Connect(host, port, CSocket::unspec, bindAddress);
+	if (res && res != EINPROGRESS) {
 		ResetSocket();
 		return false;
 	}
@@ -573,11 +547,10 @@ void CTransferSocket::SetActive()
 {
 	if (m_transferEndReason != TransferEndReason::none)
 		return;
-	if (m_transferMode == TransferMode::download || m_transferMode == TransferMode::upload)
-	{
-		CFtpFileTransferOpData *pData = static_cast<CFtpFileTransferOpData *>(static_cast<CRawTransferOpData *>(m_pControlSocket->m_pCurOpData)->pOldData);
-		if (pData && pData->pIOThread)
-			pData->pIOThread->SetEventHandler(this);
+	if (m_transferMode == TransferMode::download || m_transferMode == TransferMode::upload) {
+		if (ioThread_) {
+			ioThread_->SetEventHandler(this);
+		}
 	}
 
 	m_bActive = true;
@@ -590,7 +563,7 @@ void CTransferSocket::SetActive()
 
 void CTransferSocket::TransferEnd(TransferEndReason reason)
 {
-	m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::TransferEnd(%d)"), reason);
+	controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("CTransferSocket::TransferEnd(%d)"), reason);
 
 	if (m_transferEndReason != TransferEndReason::none)
 		return;
@@ -598,15 +571,15 @@ void CTransferSocket::TransferEnd(TransferEndReason reason)
 
 	ResetSocket();
 
-	m_pEngine->SendEvent(engineTransferEnd);
+	engine_.SendEvent<CFileZillaEngineEvent>(engineTransferEnd);
 }
 
 CSocket* CTransferSocket::CreateSocketServer(int port)
 {
 	CSocket* pServer = new CSocket(this);
-	int res = pServer->Listen(m_pControlSocket->m_pSocket->GetAddressFamily(), port);
+	int res = pServer->Listen(controlSocket_.m_pSocket->GetAddressFamily(), port);
 	if (res) {
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("Could not listen on port %d: %s"), port, CSocket::GetErrorDescription(res));
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Could not listen on port %d: %s"), port, CSocket::GetErrorDescription(res));
 		delete pServer;
 		return 0;
 	}
@@ -618,8 +591,7 @@ CSocket* CTransferSocket::CreateSocketServer(int port)
 
 CSocket* CTransferSocket::CreateSocketServer()
 {
-	if (!m_pEngine->GetOptions()->GetOptionVal(OPTION_LIMITPORTS))
-	{
+	if (!engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS)) {
 		// Ask the systen for a port
 		CSocket* pServer = CreateSocketServer(0);
 		return pServer;
@@ -637,13 +609,12 @@ CSocket* CTransferSocket::CreateSocketServer()
 
 	static int start = 0;
 
-	int low = m_pEngine->GetOptions()->GetOptionVal(OPTION_LIMITPORTS_LOW);
-	int high = m_pEngine->GetOptions()->GetOptionVal(OPTION_LIMITPORTS_HIGH);
+	int low = engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS_LOW);
+	int high = engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS_HIGH);
 	if (low > high)
 		low = high;
 
-	if (start < low || start > high)
-	{
+	if (start < low || start > high) {
 		start = GetRandomNumber(low, high);
 		wxASSERT(start >= low && start <= high);
 	}
@@ -651,8 +622,7 @@ CSocket* CTransferSocket::CreateSocketServer()
 	CSocket* pServer = 0;
 
 	int count = high - low + 1;
-	while (count--)
-	{
+	while (count--) {
 		pServer = CreateSocketServer(start++);
 		if (pServer)
 			break;
@@ -665,18 +635,17 @@ CSocket* CTransferSocket::CreateSocketServer()
 
 bool CTransferSocket::CheckGetNextWriteBuffer()
 {
-	CFtpFileTransferOpData *pData = static_cast<CFtpFileTransferOpData *>(static_cast<CRawTransferOpData *>(m_pControlSocket->m_pCurOpData)->pOldData);
 	if (!m_transferBufferLen) {
-		int res = pData->pIOThread->GetNextWriteBuffer(&m_pTransferBuffer);
+		int res = ioThread_->GetNextWriteBuffer(&m_pTransferBuffer);
 
 		if (res == IO_Again)
 			return false;
 		else if (res == IO_Error) {
-			wxString error = pData->pIOThread->GetError();
+			wxString error = ioThread_->GetError();
 			if (error.empty() )
-				m_pControlSocket->LogMessage(MessageType::Error, _("Can't write data to file."));
+				controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file."));
 			else
-				m_pControlSocket->LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
+				controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
 			TransferEnd(TransferEndReason::transfer_failure_critical);
 			return false;
 		}
@@ -689,26 +658,21 @@ bool CTransferSocket::CheckGetNextWriteBuffer()
 
 bool CTransferSocket::CheckGetNextReadBuffer()
 {
-	CFtpFileTransferOpData *pData = static_cast<CFtpFileTransferOpData *>(static_cast<CRawTransferOpData *>(m_pControlSocket->m_pCurOpData)->pOldData);
 	if (!m_transferBufferLen) {
-		int res = pData->pIOThread->GetNextReadBuffer(&m_pTransferBuffer);
+		int res = ioThread_->GetNextReadBuffer(&m_pTransferBuffer);
 		if (res == IO_Again)
 			return false;
-		else if (res == IO_Error)
-		{
-			m_pControlSocket->LogMessage(MessageType::Error, _("Can't read from file"));
+		else if (res == IO_Error) {
+			controlSocket_.LogMessage(MessageType::Error, _("Can't read from file"));
 			TransferEnd(TransferEndReason::transfer_failure);
 			return false;
 		}
-		else if (res == IO_Success)
-		{
-			if (m_pTlsSocket)
-			{
+		else if (res == IO_Success) {
+			if (m_pTlsSocket) {
 				m_shutdown = true;
 
 				int error = m_pTlsSocket->Shutdown();
-				if (error != 0)
-				{
+				if (error != 0) {
 					if (error != EAGAIN)
 						TransferEnd(TransferEndReason::transfer_failure);
 					return false;
@@ -723,7 +687,7 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 	return true;
 }
 
-void CTransferSocket::OnIOThreadEvent(CIOThreadEvent&)
+void CTransferSocket::OnIOThreadEvent()
 {
 	if (!m_bActive || m_transferEndReason != TransferEndReason::none)
 		return;
@@ -736,20 +700,18 @@ void CTransferSocket::OnIOThreadEvent(CIOThreadEvent&)
 
 void CTransferSocket::FinalizeWrite()
 {
-	CFtpFileTransferOpData *pData = static_cast<CFtpFileTransferOpData *>(static_cast<CRawTransferOpData *>(m_pControlSocket->m_pCurOpData)->pOldData);
-	bool res = pData->pIOThread->Finalize(BUFFERSIZE - m_transferBufferLen);
+	bool res = ioThread_->Finalize(BUFFERSIZE - m_transferBufferLen);
 	if (m_transferEndReason != TransferEndReason::none)
 		return;
 
 	if (res)
 		TransferEnd(TransferEndReason::successful);
-	else
-	{
-		wxString error = pData->pIOThread->GetError();
+	else {
+		wxString error = ioThread_->GetError();
 		if (error.empty())
-			m_pControlSocket->LogMessage(MessageType::Error, _("Can't write data to file."));
+			controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file."));
 		else
-			m_pControlSocket->LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
+			controlSocket_.LogMessage(MessageType::Error, _("Can't write data to file: %s"), error);
 		TransferEnd(TransferEndReason::transfer_failure_critical);
 	}
 }
@@ -760,7 +722,7 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 	m_pSocket->SetFlags(m_pSocket->GetFlags() | CSocket::flag_nodelay);
 
 	wxASSERT(!m_pBackend);
-	m_pTlsSocket = new CTlsSocket(this, m_pSocket, m_pControlSocket);
+	m_pTlsSocket = new CTlsSocket(this, m_pSocket, &controlSocket_);
 
 	if (!m_pTlsSocket->Init()) {
 		delete m_pTlsSocket;
@@ -768,7 +730,7 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 		return false;
 	}
 
-	bool try_resume = CServerCapabilities::GetCapability(*m_pControlSocket->m_pCurrentServer, tls_resume) != no;
+	bool try_resume = CServerCapabilities::GetCapability(*controlSocket_.m_pCurrentServer, tls_resume) != no;
 
 	int res = m_pTlsSocket->Handshake(pPrimaryTlsSocket, try_resume);
 	if (res && res != FZ_REPLY_WOULDBLOCK)
@@ -787,17 +749,15 @@ void CTransferSocket::TriggerPostponedEvents()
 {
 	wxASSERT(m_bActive);
 
-	if (m_postponedReceive)
-	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("Executing postponed receive"));
+	if (m_postponedReceive) {
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Executing postponed receive"));
 		m_postponedReceive = false;
 		OnReceive();
 		if (m_transferEndReason != TransferEndReason::none)
 			return;
 	}
-	if (m_postponedSend)
-	{
-		m_pControlSocket->LogMessage(MessageType::Debug_Verbose, _T("Executing postponed send"));
+	if (m_postponedSend) {
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, _T("Executing postponed send"));
 		m_postponedSend = false;
 		OnSend();
 		if (m_transferEndReason != TransferEndReason::none)
@@ -812,13 +772,12 @@ bool CTransferSocket::InitBackend()
 	if (m_pBackend)
 		return true;
 
-	if (m_pControlSocket->m_protectDataChannel)
-	{
-		if (!InitTls(m_pControlSocket->m_pTlsSocket))
+	if (controlSocket_.m_protectDataChannel) {
+		if (!InitTls(controlSocket_.m_pTlsSocket))
 			return false;
 	}
 	else
-		m_pBackend = new CSocketBackend(this, m_pSocket);
+		m_pBackend = new CSocketBackend(this, *m_pSocket, engine_.GetRateLimiter());
 
 	return true;
 }
@@ -827,7 +786,14 @@ void CTransferSocket::SetSocketBufferSizes(CSocket* pSocket)
 {
 	wxCHECK_RET(pSocket, _("SetSocketBufferSize called without socket"));
 
-	const int size_read = m_pEngine->GetOptions()->GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
-	const int size_write = m_pEngine->GetOptions()->GetOptionVal(OPTION_SOCKET_BUFFERSIZE_SEND);
+	const int size_read = engine_.GetOptions().GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
+	const int size_write = engine_.GetOptions().GetOptionVal(OPTION_SOCKET_BUFFERSIZE_SEND);
 	pSocket->SetBufferSizes(size_read, size_write);
 }
+
+void CTransferSocket::operator()(CEventBase const& ev)
+{
+	Dispatch<CSocketEvent, CIOThreadEvent>(ev, this,
+		&CTransferSocket::OnSocketEvent,
+		&CTransferSocket::OnIOThreadEvent);
+}
