@@ -84,6 +84,8 @@ public:
 
 	void ClearFindMap();
 
+	CMonotonicClock m_firstListTime;
+
 	enum
 	{
 		unsure_file_added = 0x01,
@@ -119,9 +121,7 @@ public:
 	bool has_perms() const { return (m_flags & listing_has_perms) != 0; }
 	bool has_usergroup() const { return (m_flags & listing_has_usergroup) != 0; }
 
-	CMonotonicTime m_firstListTime;
-
-	void Assign(const std::list<CDirentry> &entries);
+	void Assign(std::deque<CRefcountObject<CDirentry>> & entries);
 
 	bool RemoveEntry(unsigned int index);
 
