@@ -18,21 +18,17 @@
 #endif
 #include "edithandler.h"
 #include "dragdropmanager.h"
+#include "drop_target_ex.h"
 #include "local_filesys.h"
 #include "filelist_statusbar.h"
 #include "sizeformatting.h"
 #include "timeformatting.h"
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
-
-
-class CLocalListViewDropTarget : public CListCtrlDropTarget
+class CLocalListViewDropTarget : public CScrollableDropTarget<wxListCtrlEx>
 {
 public:
 	CLocalListViewDropTarget(CLocalListView* pLocalListView)
-		: CListCtrlDropTarget(pLocalListView)
+		: CScrollableDropTarget<wxListCtrlEx>(pLocalListView)
 		, m_pLocalListView(pLocalListView), m_pFileDataObject(new wxFileDataObject()),
 		m_pRemoteDataObject(new CRemoteDataObject())
 	{
@@ -58,13 +54,12 @@ public:
 
 	virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxListCtrlEx>::FixupDragResult(def);
+
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
 			return def;
-		if( def == wxDragLink ) {
-			def = wxDragCopy;
-		}
 
 		if (m_pLocalListView->m_fileData.empty())
 			return wxDragError;
@@ -124,7 +119,7 @@ public:
 
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
-		CListCtrlDropTarget::OnDrop(x, y);
+		CScrollableDropTarget<wxListCtrlEx>::OnDrop(x, y);
 		ClearDropHighlight();
 
 		if (m_pLocalListView->m_fileData.empty())
@@ -133,9 +128,10 @@ public:
 		return true;
 	}
 
-	virtual void DisplayDropHighlight(wxPoint point)
+	virtual int DisplayDropHighlight(wxPoint point)
 	{
 		DoDisplayDropHighlight(point);
+		return -1;
 	}
 
 	virtual wxString DoDisplayDropHighlight(wxPoint point)
@@ -185,7 +181,7 @@ public:
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
-		CListCtrlDropTarget::OnDragOver(x, y, def);
+		def = CScrollableDropTarget<wxListCtrlEx>::OnDragOver(x, y, def);
 
 		if (def == wxDragError ||
 			def == wxDragNone ||
@@ -204,8 +200,7 @@ public:
 		const wxString& subdir = DoDisplayDropHighlight(wxPoint(x, y));
 
 		CLocalPath dir = m_pLocalListView->m_pState->GetLocalDir();
-		if (subdir.empty())
-		{
+		if (subdir.empty()) {
 			const CDragDropManager* pDragDropManager = CDragDropManager::Get();
 			if (pDragDropManager && pDragDropManager->localParent == m_pLocalListView->m_dir)
 				return wxDragNone;
@@ -219,21 +214,18 @@ public:
 		if (!dir.IsWriteable())
 			return wxDragNone;
 
-		if (def == wxDragLink)
-			def = wxDragCopy;
-
 		return def;
 	}
 
 	virtual void OnLeave()
 	{
-		CListCtrlDropTarget::OnLeave();
+		CScrollableDropTarget<wxListCtrlEx>::OnLeave();
 		ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
 	{
-		CListCtrlDropTarget::OnEnter(x, y, def);
+		def = CScrollableDropTarget<wxListCtrlEx>::OnEnter(x, y, def);
 		return OnDragOver(x, y, def);
 	}
 
@@ -310,7 +302,7 @@ CLocalListView::~CLocalListView()
 #endif
 }
 
-bool CLocalListView::DisplayDir(wxString dirname)
+bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 {
 	CancelLabelEdit();
 
@@ -348,34 +340,29 @@ bool CLocalListView::DisplayDir(wxString dirname)
 	m_fileData.clear();
 	m_indexMapping.clear();
 
-	m_hasParent = CLocalPath(dirname).HasLogicalParent();
+	m_hasParent = m_dir.HasLogicalParent();
 
-	if (m_hasParent)
-	{
+	if (m_hasParent) {
 		CLocalFileData data;
 		data.dir = true;
 		data.name = _T("..");
-#ifdef __WXMSW__
-		data.label = _T("..");
-#endif
 		data.size = -1;
 		m_fileData.push_back(data);
 		m_indexMapping.push_back(0);
 	}
 
 #ifdef __WXMSW__
-	if (dirname == _T("\\"))
-	{
+	if (m_dir.GetPath() == _T("\\")) {
 		DisplayDrives();
 	}
-	else if (dirname.Left(2) == _T("\\\\"))
+	else if (m_dir.GetPath().Left(2) == _T("\\\\"))
 	{
-		int pos = dirname.Mid(2).Find('\\');
-		if (pos != -1 && pos + 3 != (int)dirname.Len())
+		int pos = m_dir.GetPath().Mid(2).Find('\\');
+		if (pos != -1 && pos + 3 != (int)m_dir.GetPath().Len())
 			goto regular_dir;
 
 		// UNC path without shares
-		DisplayShares(dirname);
+		DisplayShares(m_dir.GetPath());
 	}
 	else
 #endif
@@ -386,13 +373,12 @@ regular_dir:
 		CFilterManager filter;
 		CLocalFileSystem local_filesystem;
 
-		if (!local_filesystem.BeginFindFiles(dirname, false))
-		{
+		if (!local_filesystem.BeginFindFiles(m_dir.GetPath(), false)) {
 			SetItemCount(1);
 			return false;
 		}
 
-		wxLongLong totalSize;
+		int64_t totalSize{};
 		int unknown_sizes = 0;
 		int totalFileCount = 0;
 		int totalDirCount = 0;
@@ -401,24 +387,17 @@ regular_dir:
 		int num = m_fileData.size();
 		CLocalFileData data;
 		bool wasLink;
-		while (local_filesystem.GetNextFile(data.name, wasLink, data.dir, &data.size, &data.time, &data.attributes))
-		{
-			if (data.name.empty())
-			{
+		while (local_filesystem.GetNextFile(data.name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
+			if (data.name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
-#ifdef __WXMSW__
-			data.label = data.name;
-#endif
 
 			m_fileData.push_back(data);
-			if (!filter.FilenameFiltered(data.name, dirname, data.dir, data.size, true, data.attributes, data.time))
-			{
+			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir)
 					totalDirCount++;
-				else
-				{
+				else {
 					if (data.size != -1)
 						totalSize += data.size;
 					else
@@ -436,11 +415,9 @@ regular_dir:
 			m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
 	}
 
-	if (m_dropTarget != -1)
-	{
+	if (m_dropTarget != -1) {
 		CLocalFileData* data = GetData(m_dropTarget);
-		if (!data || !data->dir)
-		{
+		if (!data || !data->dir) {
 			SetItemState(m_dropTarget, 0, wxLIST_STATE_DROPHILITED);
 			m_dropTarget = -1;
 		}
@@ -452,8 +429,7 @@ regular_dir:
 
 	SortList(-1, -1, false);
 
-	if (IsComparing())
-	{
+	if (IsComparing()) {
 		m_originalIndexMapping.clear();
 		RefreshComparison();
 	}
@@ -480,11 +456,11 @@ int CLocalListView::OnGetItemImage(long item) const
 		if (data->name != _T(".."))
 		{
 #ifdef __WXMSW__
-			if (m_dir == _T("\\"))
+			if (m_dir.GetPath() == _T("\\"))
 				path = data->name + _T("\\");
 			else
 #endif
-				path = m_dir + data->name;
+				path = m_dir.GetPath() + data->name;
 		}
 
 		icon = pThis->GetIconIndex(data->dir ? iconType::dir : iconType::file, path);
@@ -596,35 +572,31 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 
 	const bool queue_only = action == 1;
 
-	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, CLocalPath(m_dir), path, *pServer, data->size);
+	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, *pServer, data->size);
 	m_pQueue->QueueFile_Finish(true);
 }
 
-void CLocalListView::OnMenuEnter(wxCommandEvent &event)
+void CLocalListView::OnMenuEnter(wxCommandEvent &)
 {
 	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-	if (item == -1)
-	{
+	if (item == -1) {
 		wxBell();
 		return;
 	}
 
-	if (GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1)
-	{
+	if (GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) != -1) {
 		wxBell();
 		return;
 	}
 
 	CLocalFileData *data = GetData(item);
-	if (!data || !data->dir)
-	{
+	if (!data || !data->dir) {
 		wxBell();
 		return;
 	}
 
 	wxString error;
-	if (!m_pState->SetLocalDir(data->name, &error))
-	{
+	if (!m_pState->SetLocalDir(data->name, &error)) {
 		if (!error.empty())
 			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
@@ -645,7 +617,7 @@ void CLocalListView::DisplayDrives()
 
 		CLocalFileData data;
 		data.name = drive;
-		data.label = data.name;
+		data.label = CSparseOptional<wxString>(data.name);
 		data.dir = true;
 		data.size = -1;
 
@@ -678,7 +650,7 @@ void CLocalListView::DisplayShares(wxString computer)
 	DWORD read, total;
 	DWORD resume_handle = 0;
 
-	if (computer.Last() == '\\')
+	if (!computer.empty() && computer.Last() == '\\')
 		computer.RemoveLast();
 
 	int j = m_fileData.size();
@@ -701,7 +673,7 @@ void CLocalListView::DisplayShares(wxString computer)
 			CLocalFileData data;
 			data.name = p->shi1_netname;
 #ifdef __WXMSW__
-			data.label = data.name;
+			data.label = CSparseOptional<wxString>(data.name);
 #endif
 			data.dir = true;
 			data.size = -1;
@@ -841,8 +813,7 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
 	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	if (!pServer) {
 		wxBell();
 		return;
 	}
@@ -852,8 +823,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 	bool queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
 
 	long item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (!item && m_hasParent)
 			continue;
@@ -868,23 +838,22 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 			continue;
 
 		CServerPath path = m_pState->GetRemotePath();
-		if (path.empty())
-		{
+		if (path.empty()) {
 			wxBell();
 			break;
 		}
 
-		if (data->dir)
-		{
-			path.ChangePath(data->name);
+		if (data->dir) {
+			if (!path.ChangePath(data->name)) {
+				continue;
+			}
 
 			CLocalPath localPath(m_dir);
 			localPath.AddSegment(data->name);
 			m_pQueue->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, localPath, path, *pServer);
 		}
-		else
-		{
-			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, CLocalPath(m_dir), path, *pServer, data->size);
+		else {
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, *pServer, data->size);
 			added = true;
 		}
 	}
@@ -893,7 +862,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 }
 
 // Create a new Directory
-void CLocalListView::OnMenuMkdir(wxCommandEvent& event)
+void CLocalListView::OnMenuMkdir(wxCommandEvent&)
 {
 	wxString newdir = MenuMkdir();
 	if (!newdir.empty()) {
@@ -938,7 +907,7 @@ wxString CLocalListView::MenuMkdir()
 	}
 
 	wxFileName fn(dlg.GetValue(), _T(""));
-	fn.Normalize(wxPATH_NORM_ALL, m_dir);
+	fn.Normalize(wxPATH_NORM_ALL, m_dir.GetPath());
 
 	bool res;
 
@@ -953,21 +922,14 @@ wxString CLocalListView::MenuMkdir()
 	}
 
 	// Return name of the New Directory
-	// return dlg.GetValue();
-	// return fn.GetDirs().Last();
 	return fn.GetPath();
 }
 
-void CLocalListView::OnMenuDelete(wxCommandEvent& event)
+void CLocalListView::OnMenuDelete(wxCommandEvent&)
 {
-	wxString dir = m_dir;
-	if (dir.Right(1) != _T("\\") && dir.Right(1) != _T("/"))
-		dir += _T("\\");
-
 	std::list<wxString> pathsToDelete;
 	long item = -1;
-	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		if (!item && m_hasParent)
 			continue;
 
@@ -978,7 +940,7 @@ void CLocalListView::OnMenuDelete(wxCommandEvent& event)
 		if (data->comparison_flags == fill)
 			continue;
 
-		pathsToDelete.push_back(dir + data->name);
+		pathsToDelete.push_back(m_dir.GetPath() + data->name);
 	}
 	if (!CLocalFileSystem::RecursiveDelete(pathsToDelete, this))
 		wxGetApp().DisplayEncodingWarning();
@@ -986,7 +948,7 @@ void CLocalListView::OnMenuDelete(wxCommandEvent& event)
 	m_pState->SetLocalDir(m_dir);
 }
 
-void CLocalListView::OnMenuRename(wxCommandEvent& event)
+void CLocalListView::OnMenuRename(wxCommandEvent&)
 {
 	if (GetEditControl()) {
 		GetEditControl()->SetFocus();
@@ -1022,10 +984,8 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 #endif
 
 	const int code = event.GetKeyCode();
-	if (code == WXK_DELETE || code == WXK_NUMPAD_DELETE)
-	{
-		if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
-		{
+	if (code == WXK_DELETE || code == WXK_NUMPAD_DELETE) {
+		if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1) {
 			wxBell();
 			return;
 		}
@@ -1033,22 +993,23 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 		wxCommandEvent tmp;
 		OnMenuDelete(tmp);
 	}
-	else if (code == WXK_F2)
-	{
+	else if (code == WXK_F2) {
 		wxCommandEvent tmp;
 		OnMenuRename(tmp);
 	}
-	else if (code == WXK_RIGHT && event.GetModifiers() == CursorModifierKey)
-	{
+	else if (code == WXK_RIGHT && event.GetModifiers() == CursorModifierKey) {
 		wxListEvent evt;
 		evt.m_itemIndex = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
 		OnItemActivated(evt);
 	}
-	else if (code == WXK_DOWN && event.GetModifiers() == CursorModifierKey)
-	{
+	else if (code == WXK_DOWN && event.GetModifiers() == CursorModifierKey) {
 		wxCommandEvent cmdEvent;
 		OnMenuUpload(cmdEvent);
 	}
+	else if (code == 'N' && event.GetModifiers() == (wxMOD_CONTROL | wxMOD_SHIFT)) {
+		wxCommandEvent cmdEvent;
+		OnMenuMkdir(cmdEvent);
+	}
 	else
 		event.Skip();
 }
@@ -1092,12 +1053,12 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 	if (newname == data->name)
 		return false;
 
-	if (!RenameFile(this, m_dir, data->name, newname))
+	if (!RenameFile(this, m_dir.GetPath(), data->name, newname))
 		return false;
 
 	data->name = newname;
 #ifdef __WXMSW__
-	data->label = data->name;
+	data->label.clear();
 #endif
 	m_pState->RefreshLocal();
 
@@ -1121,7 +1082,7 @@ void CLocalListView::ApplyCurrentFilter()
 	if (m_pFilelistStatusBar)
 		m_pFilelistStatusBar->UnselectAll();
 
-	wxLongLong totalSize;
+	int64_t totalSize{};
 	int unknown_sizes = 0;
 	int totalFileCount = 0;
 	int totalDirCount = 0;
@@ -1130,25 +1091,23 @@ void CLocalListView::ApplyCurrentFilter()
 	m_indexMapping.clear();
 	if (m_hasParent)
 		m_indexMapping.push_back(0);
-	for (unsigned int i = min; i < m_fileData.size(); i++)
-	{
+	for (unsigned int i = min; i < m_fileData.size(); ++i) {
 		const CLocalFileData& data = m_fileData[i];
 		if (data.comparison_flags == fill)
 			continue;
-		if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.time)) {
-			hidden++;
+		if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
+			++hidden;
 			continue;
 		}
 
 		if (data.dir)
-			totalDirCount++;
-		else
-		{
+			++totalDirCount;
+		else {
 			if (data.size != -1)
 				totalSize += data.size;
 			else
-				unknown_sizes++;
-			totalFileCount++;
+				++unknown_sizes;
+			++totalFileCount;
 		}
 
 		m_indexMapping.push_back(i);
@@ -1160,8 +1119,7 @@ void CLocalListView::ApplyCurrentFilter()
 
 	SortList(-1, -1, false);
 
-	if (IsComparing())
-	{
+	if (IsComparing()) {
 		m_originalIndexMapping.clear();
 		RefreshComparison();
 	}
@@ -1281,10 +1239,10 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	}
 }
 
-void CLocalListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void*)
+void CLocalListView::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
-		DisplayDir(m_pState->GetLocalDir().GetPath());
+		DisplayDir(m_pState->GetLocalDir());
 	else if (notification == STATECHANGE_APPLYFILTER)
 		ApplyCurrentFilter();
 	else
@@ -1294,11 +1252,10 @@ void CLocalListView::OnStateChange(CState* pState, enum t_statechange_notificati
 	}
 }
 
-void CLocalListView::OnBeginDrag(wxListEvent& event)
+void CLocalListView::OnBeginDrag(wxListEvent&)
 {
 	long item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
@@ -1314,8 +1271,7 @@ void CLocalListView::OnBeginDrag(wxListEvent& event)
 	pDragDropManager->localParent = m_dir;
 
 	item = -1;
-	for (;;)
-	{
+	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 		if (item == -1)
 			break;
@@ -1327,13 +1283,12 @@ void CLocalListView::OnBeginDrag(wxListEvent& event)
 		if (data->comparison_flags == fill)
 			continue;
 
-		const wxString name = m_dir + data->name;
+		wxString const name = m_dir.GetPath() + data->name;
 		pDragDropManager->m_localFiles.push_back(name);
 		obj.AddFile(name);
 	}
 
-	if (obj.GetFilenames().empty())
-	{
+	if (obj.GetFilenames().empty()) {
 		pDragDropManager->Release();
 		return;
 	}
@@ -1348,8 +1303,7 @@ void CLocalListView::OnBeginDrag(wxListEvent& event)
 
 	pDragDropManager->Release();
 
-	if (!handled_internally && (res == wxDragCopy || res == wxDragMove))
-	{
+	if (!handled_internally && (res == wxDragCopy || res == wxDragMove)) {
 		// We only need to refresh local side if the operation got handled
 		// externally, the internal handlers do this for us already
 		m_pState->RefreshLocal();
@@ -1361,41 +1315,35 @@ void CLocalListView::RefreshFile(const wxString& file)
 	CLocalFileData data;
 
 	bool wasLink;
-	enum CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(m_dir + file, wasLink, &data.size, &data.time, &data.attributes);
+	enum CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(m_dir.GetPath() + file, wasLink, &data.size, &data.time, &data.attributes);
 	if (type == CLocalFileSystem::unknown)
 		return;
 
 	data.name = file;
-#ifdef __WXMSW__
-	data.label = file;
-#endif
 	data.dir = type == CLocalFileSystem::dir;
 
 	CFilterManager filter;
-	if (filter.FilenameFiltered(data.name, m_dir, data.dir, data.size, true, data.attributes, data.time))
+	if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time))
 		return;
 
 	CancelLabelEdit();
 
 	// Look if file data already exists
 	unsigned int i = 0;
-	for (auto iter = m_fileData.begin(); iter != m_fileData.end(); ++iter, ++i)
-	{
+	for (auto iter = m_fileData.begin(); iter != m_fileData.end(); ++iter, ++i) {
 		const CLocalFileData& oldData = *iter;
 		if (oldData.name != file)
 			continue;
 
 		// Update file list status bar
-		if (m_pFilelistStatusBar)
-		{
+		if (m_pFilelistStatusBar) {
 #ifndef __WXMSW__
 			// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
 			if (GetSelectedItemCount())
 #endif
 			{
 				int item = -1;
-				for (;;)
-				{
+				for (;;) {
 					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 					if (item == -1)
 						break;
@@ -1428,13 +1376,11 @@ void CLocalListView::RefreshFile(const wxString& file)
 		data.fileType = oldData.fileType;
 
 		*iter = data;
-		if (IsComparing())
-		{
+		if (IsComparing()) {
 			// Sort order doesn't change
 			RefreshComparison();
 		}
-		else
-		{
+		else {
 			if (m_sortColumn)
 				SortList();
 			RefreshListOnly(false);
@@ -1449,12 +1395,12 @@ void CLocalListView::RefreshFile(const wxString& file)
 
 	wxString focused;
 	std::list<wxString> selectedNames;
-	if (IsComparing())
-	{
-		wxASSERT(!m_originalIndexMapping.empty());
+	if (IsComparing()) {
 		selectedNames = RememberSelectedItems(focused);
-		m_indexMapping.clear();
-		m_originalIndexMapping.swap(m_indexMapping);
+		if (!m_originalIndexMapping.empty()) {
+			m_indexMapping.clear();
+			m_originalIndexMapping.swap(m_indexMapping);
+		}
 	}
 
 	// Insert new entry
@@ -1472,26 +1418,22 @@ void CLocalListView::RefreshFile(const wxString& file)
 	const int item = insertPos - m_indexMapping.begin();
 	m_indexMapping.insert(insertPos, index);
 
-	if (!IsComparing())
-	{
+	if (!IsComparing()) {
 		SetItemCount(m_indexMapping.size());
 
 		// Move selections
 		int prevState = 0;
-		for (unsigned int i = item; i < m_indexMapping.size(); i++)
-		{
-			int state = GetItemState(i, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED);
-			if (state != prevState)
-			{
-				SetItemState(i, prevState, wxLIST_STATE_FOCUSED);
-				SetSelection(i, (prevState & wxLIST_STATE_SELECTED) != 0);
+		for (unsigned int j = item; j < m_indexMapping.size(); ++j) {
+			int state = GetItemState(j, wxLIST_STATE_SELECTED | wxLIST_STATE_FOCUSED);
+			if (state != prevState) {
+				SetItemState(j, prevState, wxLIST_STATE_FOCUSED);
+				SetSelection(j, (prevState & wxLIST_STATE_SELECTED) != 0);
 				prevState = state;
 			}
 		}
 		RefreshListOnly();
 	}
-	else
-	{
+	else {
 		RefreshComparison();
 		if (m_pFilelistStatusBar)
 			m_pFilelistStatusBar->UnselectAll();
@@ -1588,7 +1530,7 @@ void CLocalListView::FinishComparison()
 	pOther->ScrollTopItem(GetTopItem());
 }
 
-bool CLocalListView::CanStartComparison(wxString* pError)
+bool CLocalListView::CanStartComparison()
 {
 	return true;
 }
@@ -1599,23 +1541,20 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 	if (!data)
 		return wxString();
 
-	if (!column)
-	{
+	if (!column) {
 #ifdef __WXMSW__
-		return data->label;
+		return data->label ? *data->label : data->name;
 #else
 		return data->name;
 #endif
 	}
-	else if (column == 1)
-	{
+	else if (column == 1) {
 		if (data->size < 0)
 			return wxString();
 		else
 			return CSizeFormat::Format(data->size);
 	}
-	else if (column == 2)
-	{
+	else if (column == 2) {
 		if (!item && m_hasParent)
 			return wxString();
 
@@ -1623,7 +1562,7 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 			return wxString();
 
 		if (data->fileType.empty())
-			data->fileType = GetType(data->name, data->dir, m_dir);
+			data->fileType = GetType(data->name, data->dir, m_dir.GetPath());
 
 		return data->fileType;
 	}
@@ -1633,38 +1572,32 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 	return wxString();
 }
 
-void CLocalListView::OnMenuEdit(wxCommandEvent& event)
+void CLocalListView::OnMenuEdit(wxCommandEvent&)
 {
 	CServer server;
 	CServerPath path;
 
-	if (!m_pState->GetServer())
-	{
-		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL))
-		{
+	if (!m_pState->GetServer()) {
+		if (COptions::Get()->GetOptionVal(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
-	else
-	{
+	else {
 		server = *m_pState->GetServer();
 
 		path = m_pState->GetRemotePath();
-		if (path.empty())
-		{
+		if (path.empty()) {
 			wxMessageBoxEx(_("Cannot edit file, remote path unknown."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
 
-	std::list<CLocalFileData> selected_item_list;
+	std::vector<CEditHandler::FileData> selected_item;
 
 	long item = -1;
-	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
-		if (!item && m_hasParent)
-		{
+	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
+		if (!item && m_hasParent) {
 			wxBell();
 			return;
 		}
@@ -1673,8 +1606,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 		if (!data)
 			continue;
 
-		if (data->dir)
-		{
+		if (data->dir) {
 			wxBell();
 			return;
 		}
@@ -1682,99 +1614,18 @@ void CLocalListView::OnMenuEdit(wxCommandEvent& event)
 		if (data->comparison_flags == fill)
 			continue;
 
-		selected_item_list.push_back(*data);
+		selected_item.push_back({m_dir.GetPath() + data->name, data->size});
 	}
 
 	CEditHandler* pEditHandler = CEditHandler::Get();
-
-	if (selected_item_list.empty())
-	{
-		wxBell();
-		return;
-	}
-
-	if (selected_item_list.size() > 10)
-	{
-
-		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
-		dlg.SetTitle(_("Confirmation needed"));
-		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
-
-		if (!dlg.Run())
-			return;
-	}
-
-	for (std::list<CLocalFileData>::const_iterator data = selected_item_list.begin(); data != selected_item_list.end(); ++data)
-	{
-		wxFileName fn(m_dir, data->name);
-
-		bool dangerous = false;
-		bool program_exists = false;
-		wxString cmd = pEditHandler->CanOpen(CEditHandler::local, fn.GetFullPath(), dangerous, program_exists);
-		if (cmd.empty())
-		{
-			CNewAssociationDialog dlg(this);
-			if (!dlg.Run(fn.GetFullName()))
-				continue;
-			cmd = pEditHandler->CanOpen(CEditHandler::local, fn.GetFullPath(), dangerous, program_exists);
-			if (cmd.empty())
-			{
-				wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
-				continue;
-			}
-		}
-		if (!program_exists)
-		{
-			wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), fn.GetFullPath(), cmd);
-			wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
-			continue;
-		}
-		if (dangerous)
-		{
-			int res = wxMessageBoxEx(_("The selected file would be executed directly.\nThis can be dangerous and damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
-			if (res != wxYES)
-			{
-				wxBell();
-				continue;
-			}
-		}
-
-		CEditHandler::fileState state = pEditHandler->GetFileState(fn.GetFullPath());
-		switch (state)
-		{
-		case CEditHandler::upload:
-		case CEditHandler::upload_and_remove:
-			wxMessageBoxEx(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
-			continue;
-		case CEditHandler::edit:
-			{
-				int res = wxMessageBoxEx(wxString::Format(_("A file with that name is already being edited. Do you want to reopen '%s'?"), fn.GetFullPath()), _("Selected file already being edited"), wxICON_QUESTION | wxYES_NO);
-				if (res != wxYES)
-				{
-					wxBell();
-					continue;
-				}
-				pEditHandler->StartEditing(fn.GetFullPath());
-				continue;
-			}
-		default:
-			break;
-		}
-
-		wxString file = fn.GetFullPath();
-		if (!pEditHandler->AddFile(CEditHandler::local, file, path, server))
-		{
-			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
-			continue;
-		}
-	}
+	pEditHandler->Edit(CEditHandler::local, selected_item, path, server, this);
 }
 
-void CLocalListView::OnMenuOpen(wxCommandEvent& event)
+void CLocalListView::OnMenuOpen(wxCommandEvent&)
 {
 	long item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	if (item == -1) {
-		OpenInFileManager(m_dir);
+		OpenInFileManager(m_dir.GetPath());
 		return;
 	}
 
@@ -1809,7 +1660,6 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 	}
 
 	if (selected_item_list.size() > 10) {
-
 		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
 		dlg.SetTitle(_("Confirmation needed"));
 		dlg.AddText(_("You have selected more than 10 files or directories to open, do you really want to continue?"));
@@ -1830,7 +1680,7 @@ void CLocalListView::OnMenuOpen(wxCommandEvent& event)
 			continue;
 		}
 
-		wxFileName fn(m_dir, data.name);
+		wxFileName fn(m_dir.GetPath(), data.name);
 		if (wxLaunchDefaultApplication(fn.GetFullPath(), 0)) {
 			continue;
 		}
@@ -1863,7 +1713,7 @@ bool CLocalListView::ItemIsDir(int index) const
 	return m_fileData[index].dir;
 }
 
-wxLongLong CLocalListView::ItemGetSize(int index) const
+int64_t CLocalListView::ItemGetSize(int index) const
 {
 	return m_fileData[index].size;
 }
@@ -1878,22 +1728,19 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
 	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
 
-	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED)
-	{
+	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED) {
 		delete m_pVolumeEnumeratorThread;
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	if (m_dir != _T("\\"))
+	if (m_dir.GetPath() != _T("\\"))
 		return;
 
-	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter)
-	{
+	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
 		wxString drive = iter->volume;
 
 		unsigned int item, index;
-		for (item = 1; item < m_indexMapping.size(); ++item)
-		{
+		for (item = 1; item < m_indexMapping.size(); ++item) {
 			index = m_indexMapping[item];
 			if (m_fileData[index].name == drive || m_fileData[index].name.Left(drive.Len() + 1) == drive + _T(" "))
 				break;
@@ -1901,7 +1748,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		if (item >= m_indexMapping.size())
 			continue;
 
-		m_fileData[index].label = drive + _T(" (") + iter->volumeName + _T(")");
+		m_fileData[index].label = CSparseOptional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
 
 		RefreshItem(item);
 	}
@@ -1909,7 +1756,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 
 #endif
 
-void CLocalListView::OnMenuRefresh(wxCommandEvent& event)
+void CLocalListView::OnMenuRefresh(wxCommandEvent&)
 {
 	m_pState->RefreshLocal();
 }
