@@ -3,6 +3,12 @@
 
 #include "local_path.h"
 
+#include <option_change_event_handler.h>
+
+#include <mutex.h>
+
+#include <wx/timer.h>
+
 enum interfaceOptions
 {
 	OPTION_NUMTRANSFERS = OPTIONS_ENGINE_NUM,
@@ -20,7 +26,6 @@ enum interfaceOptions
 	OPTION_UPDATECHECK_LASTDATE,
 	OPTION_UPDATECHECK_NEWVERSION,
 	OPTION_UPDATECHECK_CHECKBETA,
-	OPTION_UPDATECHECK_DOWNLOADDIR,
 	OPTION_DEBUG_MENU,
 	OPTION_FILEEXISTS_DOWNLOAD,
 	OPTION_FILEEXISTS_UPLOAD,
@@ -86,6 +91,7 @@ enum interfaceOptions
 	OPTION_STRIP_VMS_REVISION,
 	OPTION_INTERFACE_SITEMANAGER_ON_STARTUP,
 	OPTION_PROMPTPASSWORDSAVE,
+	OPTION_PERSISTENT_CHOICES,
 
 	// Default/internal options
 	OPTION_DEFAULT_SETTINGSDIR, // guaranteed to be (back)slash-terminated
@@ -100,8 +106,8 @@ struct t_OptionsCache
 {
 	bool operator==(wxString const& v) const { return strValue == v; }
 	bool operator==(int v) const { return numValue == v; }
-	t_OptionsCache& operator=(wxString const& v) { strValue = v; return *this; }
-	t_OptionsCache& operator=(int v) { numValue = v; return *this; }
+	t_OptionsCache& operator=(wxString const& v);
+	t_OptionsCache& operator=(int v);
 
 	bool from_default;
 	int numValue;
@@ -151,15 +157,17 @@ protected:
 
 	TiXmlElement* CreateSettingsXmlElement();
 
-	std::map<std::string, int> GetNameOptionMap() const;
-	void LoadOptions(std::map<std::string, int> const& nameOptionMap, TiXmlElement* settings = 0);
-	void LoadGlobalDefaultOptions(std::map<std::string, int> const& nameOptionMap);
-	void LoadOptionFromElement(TiXmlElement* pOption, std::map<std::string, int> const& nameOptionMap, bool allowDefault);
+	std::map<std::string, unsigned int> GetNameOptionMap() const;
+	void LoadOptions(std::map<std::string, unsigned int> const& nameOptionMap, TiXmlElement* settings = 0);
+	void LoadGlobalDefaultOptions(std::map<std::string, unsigned int> const& nameOptionMap);
+	void LoadOptionFromElement(TiXmlElement* pOption, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault);
 	CLocalPath InitSettingsDir();
 	void SetDefaultValues();
 
 	void Save();
 
+	void NotifyChangedOptions();
+
 	CXmlFile* m_pXmlFile;
 
 	t_OptionsCache m_optionsCache[OPTIONS_NUM];
@@ -172,6 +180,10 @@ protected:
 
 	DECLARE_EVENT_TABLE()
 	void OnTimer(wxTimerEvent& event);
+
+	mutex m_sync_;
+
+	changed_options_t changedOptions_;
 };
 
 #endif //__OPTIONS_H__
