@@ -5,14 +5,16 @@
 #include "richedit.h"
 #endif
 
+#include <wx/timer.h>
+
 class CFastTextCtrl;
-class CStatusView : public wxNavigationEnabled<wxWindow>
+class CStatusView final : public wxNavigationEnabled<wxWindow>
 {
 public:
 	CStatusView(wxWindow* parent, wxWindowID id);
 	virtual ~CStatusView();
 
-	void AddToLog(CLogmsgNotification *pNotification);
+	void AddToLog(CLogmsgNotification const& pNotification);
 	void AddToLog(MessageType messagetype, const wxString& message, const wxDateTime& time);
 
 	void InitDefAttr();
