@@ -25,15 +25,8 @@ CToolBar::~CToolBar()
 CToolBar* CToolBar::Load(CMainFrame* pMainFrame)
 {
 	{
-		wxSize iconSize;
 		wxString str = COptions::Get()->GetOption(OPTION_THEME_ICONSIZE);
-		if (str == _T("32x32"))
-			iconSize = CThemeProvider::GetIconSize(iconSizeNormal);
-		else if (str == _T("48x48"))
-			iconSize = CThemeProvider::GetIconSize(iconSizeLarge);
-		else
-			iconSize = CThemeProvider::GetIconSize(iconSizeSmall);
-
+		wxSize iconSize = CThemeProvider::GetIconSize(str);
 		wxToolBarXmlHandlerEx::SetIconSize(iconSize);
 	}
 
@@ -43,14 +36,14 @@ CToolBar* CToolBar::Load(CMainFrame* pMainFrame)
 
 	toolbar->m_pMainFrame = pMainFrame;
 
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_REMOTE_IDLE, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SERVER, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SYNC_BROWSE, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_COMPARISON, true, true);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_APPLYFILTER, true, false);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_REMOTE_IDLE, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SERVER, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_SYNC_BROWSE, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_COMPARISON, true);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_APPLYFILTER, true);
 
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_QUEUEPROCESSING, false, false);
-	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_CHANGEDCONTEXT, false, false);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_QUEUEPROCESSING, false);
+	CContextManager::Get()->RegisterHandler(toolbar, STATECHANGE_CHANGEDCONTEXT, false);
 
 	toolbar->RegisterOption(OPTION_SHOW_MESSAGELOG);
 	toolbar->RegisterOption(OPTION_SHOW_QUEUE);
@@ -58,10 +51,6 @@ CToolBar* CToolBar::Load(CMainFrame* pMainFrame)
 	toolbar->RegisterOption(OPTION_SHOW_TREE_REMOTE);
 	toolbar->RegisterOption(OPTION_MESSAGELOG_POSITION);
 
-#if defined(EVT_TOOL_DROPDOWN) && defined(__WXMSW__)
-	toolbar->MakeDropdownTool(XRCID("ID_TOOLBAR_SITEMANAGER"));
-#endif
-
 #ifdef __WXMSW__
 	int majorVersion, minorVersion;
 	wxGetOsVersion(& majorVersion, & minorVersion);
@@ -133,7 +122,6 @@ void CToolBar::UpdateToolbarState()
 
 	EnableTool(XRCID("ID_TOOLBAR_DISCONNECT"), pServer && idle);
 	EnableTool(XRCID("ID_TOOLBAR_CANCEL"), pServer && !idle);
-	EnableTool(XRCID("ID_TOOLBAR_COMPARISON"), pServer != 0);
 	EnableTool(XRCID("ID_TOOLBAR_SYNCHRONIZED_BROWSING"), pServer != 0);
 	EnableTool(XRCID("ID_TOOLBAR_FIND"), pServer && idle);
 
@@ -143,72 +131,43 @@ void CToolBar::UpdateToolbarState()
 	bool canReconnect;
 	if (pServer || !idle)
 		canReconnect = false;
-	else
-	{
+	else {
 		CServer tmp;
 		canReconnect = !pState->GetLastServer().GetHost().empty();
 	}
 	EnableTool(XRCID("ID_TOOLBAR_RECONNECT"), canReconnect);
 }
 
-void CToolBar::OnOptionChanged(int option)
+void CToolBar::OnOptionsChanged(changed_options_t const& options)
 {
-	switch (option)
-	{
-	case OPTION_SHOW_MESSAGELOG:
+	if (options.test(OPTION_SHOW_MESSAGELOG)) {
 		ToggleTool(XRCID("ID_TOOLBAR_LOGVIEW"), COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG) != 0);
-		break;
-	case OPTION_SHOW_QUEUE:
+	}
+	if (options.test(OPTION_SHOW_QUEUE)) {
 		ToggleTool(XRCID("ID_TOOLBAR_QUEUEVIEW"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE) != 0);
-		break;
-	case OPTION_SHOW_TREE_LOCAL:
+	}
+	if (options.test(OPTION_SHOW_TREE_LOCAL)) {
 		ToggleTool(XRCID("ID_TOOLBAR_LOCALTREEVIEW"), COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL) != 0);
-		break;
-	case OPTION_SHOW_TREE_REMOTE:
+	}
+	if (options.test(OPTION_SHOW_TREE_REMOTE)) {
 		ToggleTool(XRCID("ID_TOOLBAR_REMOTETREEVIEW"), COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE) != 0);
-		break;
-	case OPTION_MESSAGELOG_POSITION:
+	}
+	if (options.test(OPTION_MESSAGELOG_POSITION)) {
 		if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2)
 			HideTool(XRCID("ID_TOOLBAR_LOGVIEW"));
-		else
-		{
+		else {
 			ShowTool(XRCID("ID_TOOLBAR_LOGVIEW"));
 			ToggleTool(XRCID("ID_TOOLBAR_LOGVIEW"), COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG) != 0);
 		}
-		break;
-	default:
-		break;
 	}
 }
 
-#if defined(EVT_TOOL_DROPDOWN) && defined(__WXMSW__)
-void CToolBar::MakeDropdownTool(int id)
-{
-	wxToolBarToolBase* pOldTool = FindById(id);
-	if (!pOldTool)
-		return;
-
-	wxToolBarToolBase* pTool = new wxToolBarToolBase(0, id,
-		pOldTool->GetLabel(), pOldTool->GetNormalBitmap(), pOldTool->GetDisabledBitmap(),
-		wxITEM_DROPDOWN, NULL, pOldTool->GetShortHelp(), pOldTool->GetLongHelp());
-
-	int pos = GetToolPos(id);
-	wxASSERT(pos != wxNOT_FOUND);
-
-	DeleteToolByPos(pos);
-	InsertTool(pos, pTool);
-	Realize();
-}
-#endif
-
 bool CToolBar::ShowTool(int id)
 {
 	int offset = 0;
 
-	for (auto iter = m_hidden_tools.begin(); iter != m_hidden_tools.end(); ++iter)
-	{
-		if (iter->second->GetId() != id)
-		{
+	for (auto iter = m_hidden_tools.begin(); iter != m_hidden_tools.end(); ++iter) {
+		if (iter->second->GetId() != id) {
 			offset++;
 			continue;
 		}
@@ -233,12 +192,11 @@ bool CToolBar::HideTool(int id)
 	if (!tool)
 		return false;
 
-	for (auto iter = m_hidden_tools.begin(); iter != m_hidden_tools.end(); ++iter)
-	{
-		if (iter->first > pos)
+	for (auto const& iter : m_hidden_tools) {
+		if (iter.first > pos)
 			break;
 
-		pos++;
+		++pos;
 	}
 
 	m_hidden_tools[pos] = tool;
