@@ -34,18 +34,12 @@ bool CWrapEngine::CanWrapBefore(const wxChar& c)
 {
 	// Check if this is a punctuation character, we're not allowed
 	// to wrap before such a character
-	const wxChar* p = m_noWrapChars;
-	while (*p)
-	{
-		if (*p == c)
-			break;
-
-		p++;
+	for (auto p = m_noWrapChars; p && *p; ++p) {
+		if (*p == c) {
+			return false;
+		}
 	}
-	if (!*p)
-		return true;
-
-	return false;
+	return true;
 }
 
 bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned long maxLength)
@@ -53,8 +47,8 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 	WRAPASSERT(text.Find(_T("  ")) == -1);
 	WRAPASSERT(text.Find(_T(" \n")) == -1);
 	WRAPASSERT(text.Find(_T("\n ")) == -1);
-	WRAPASSERT(text.Last() != ' ');
-	WRAPASSERT(text.Last() != '\n');
+	WRAPASSERT(text.empty() || text.Last() != ' ');
+	WRAPASSERT(text.empty() || text.Last() != '\n');
 
 	// See comment at start of WrapText function what this function does
 	wxString wrappedText;
@@ -127,7 +121,7 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 			if (lineLength > maxLength && wrappable)
 			{
 				wxString tmp = wxString(str, wrappable - str);
-				if (tmp.Last() == ' ')
+				if (!tmp.empty() && tmp.Last() == ' ')
 					tmp.RemoveLast();
 				wrappedText += tmp + _T("\n");
 				if (*wrappable != ' ')
@@ -163,8 +157,8 @@ bool CWrapEngine::WrapTextChinese(wxWindow* parent, wxString &text, unsigned lon
 	wxASSERT(temp.Find(_T("  ")) == -1);
 	wxASSERT(temp.Find(_T(" \n")) == -1);
 	wxASSERT(temp.Find(_T("\n ")) == -1);
-	wxASSERT(temp.Last() != ' ');
-	wxASSERT(temp.Last() != '\n');
+	wxASSERT(temp.empty() || temp.Last() != ' ');
+	wxASSERT(temp.empty() || temp.Last() != '\n');
 	temp.Replace(_T("&"), _T(""));
 	while (!temp.empty())
 	{
@@ -529,20 +523,15 @@ bool CWrapEngine::WrapRecursive(wxWindow* wnd, double ratio, const char* name /*
 	return (WrapRecursive(windows, ratio, name, canvas, minRequestedSize) & wrap_failed) == 0;
 }
 
-void CWrapEngine::UnwrapRecursive_Wrapped(const std::list<int> &wrapped, std::vector<wxWindow*> &windows, bool remove_fitting /*=false*/)
+void CWrapEngine::UnwrapRecursive_Wrapped(std::vector<int> const& wrapped, std::vector<wxWindow*> &windows, bool remove_fitting /*=false*/)
 {
 	unsigned int i = 0;
-	for (std::list<int>::const_iterator iter = wrapped.begin();
-		iter != wrapped.end();
-		++iter)
-	{
+	for (auto const& wrap : wrapped) {
 		UnwrapRecursive(windows[i], windows[i]->GetSizer());
 		windows[i]->GetSizer()->Layout();
 
-		if (!(*iter & wrap_didwrap) && !(*iter & wrap_failed))
-		{
-			if (!(*iter) && remove_fitting)
-			{
+		if (!(wrap & wrap_didwrap) && !(wrap & wrap_failed)) {
+			if (!wrap && remove_fitting) {
 				// Page didn't need to be wrapped with current wrap offset,
 				// remove it since desired width will only be larger in further wrappings.
 				windows.erase(windows.begin() + i);
@@ -626,7 +615,7 @@ bool CWrapEngine::WrapRecursive(std::vector<wxWindow*>& windows, double ratio, c
 #endif
 
 	for (;;) {
-		std::list<int> didwrap;
+		std::vector<int> didwrap;
 
 		size = minRequestedSize;
 		int res = 0;
@@ -1128,8 +1117,7 @@ void CWrapEngine::CheckLanguage()
 		m_wrapOnEveryChar = true;
 		m_noWrapChars = noWrapChars_Chinese;
 	}
-	else
-	{
+	else {
 		m_wrapOnEveryChar = false;
 		m_noWrapChars = 0;
 	}
